SystemOrganization addCategory: #'Env-Tests'!
SystemOrganization addCategory: #'Env-Core'!

TestCase subclass: #EnvironmentTest
	instanceVariableNames: 'env value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Env-Tests'!

!EnvironmentTest methodsFor: 'running' stamp: 'cwp 4/22/2012 08:38'!
createClass: aSymbol
	| builder |
	builder := ClassBuilder new.
	builder
		name: aSymbol
		inEnvironment: env
		subclassOf: Object
		type: #normal
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'Test'.
! !

!EnvironmentTest methodsFor: 'running' stamp: 'cwp 4/20/2012 23:27'!
setUp
	env := Environment new.
	value := Object new.! !

!EnvironmentTest methodsFor: 'binding tests' stamp: 'cwp 4/20/2012 19:26'!
testAtDoesntFindUndeclared
	env := Environment new.
	env bindingOf: #Griffle.
	self should: [ env at: #Griffle ] raise: KeyNotFound! !

!EnvironmentTest methodsFor: 'compatibility tests' stamp: 'cwp 4/20/2012 23:27'!
testAtIfAbsent
	| result |
	result := env at: #Griffle ifAbsent: [value].
	self assert: result == value! !

!EnvironmentTest methodsFor: 'compatibility tests' stamp: 'cwp 4/20/2012 23:27'!
testAtIfAbsentNot
	| result |
	env at: #Griffle put: value.
	result := env at: #Griffle ifAbsent: [self assert: false].
	self assert: result == value! !

!EnvironmentTest methodsFor: 'compatibility tests' stamp: 'cwp 4/20/2012 23:27'!
testAtIfPresent
	| result |
	env at: #Griffle put: value.
	env at: #Griffle ifPresent: [:v | result := v].
	self assert: result == value! !

!EnvironmentTest methodsFor: 'compatibility tests' stamp: 'cwp 4/20/2012 19:28'!
testAtIfPresentNot
	env at: #Griffle ifPresent: [self assert: false].! !

!EnvironmentTest methodsFor: 'export tests' stamp: 'cwp 4/20/2012 23:27'!
testExplicitExport
	env requireExplicitExports.
	env at: #Griffle put: value.
	env export: #Griffle.
	self assert: (env exports at: #Griffle) == value! !

!EnvironmentTest methodsFor: 'export tests' stamp: 'cwp 4/20/2012 23:27'!
testImportFromOther
	| foreign |
	foreign := Environment new.
	foreign at: #Griffle put: value.
	env importEnvironment: foreign.
	self assert: (env bindingOf: #Griffle) value == value! !

!EnvironmentTest methodsFor: 'compiling tests' stamp: 'cwp 4/22/2012 08:36'!
testInternalVisibility
	| griffle plonk |
	self createClass: #Griffle.
	self createClass: #Plonk.
	griffle := env at: #Griffle.
	griffle compile: 'plonk ^ Plonk'.
	plonk := griffle new plonk.
	self assert: (env at: #Plonk) == plonk! !

!EnvironmentTest methodsFor: 'binding tests' stamp: 'cwp 4/20/2012 23:54'!
testReadOnlyBindings
	| binding |
	env at: #Griffle put: value.
	binding := env bindingOf: #Griffle.
	self
		should: [binding value: nil]
		raise: AttemptToWriteReadOnlyGlobal! !

!EnvironmentTest methodsFor: 'export tests' stamp: 'cwp 4/20/2012 23:36'!
testRequireExplicitExportIdempotency
	env requireExplicitExports.
	env at: #Griffle put: value.
	env export: #Griffle.
	env requireExplicitExports.
	self assert: (env exports includesKey: #Griffle)! !

!EnvironmentTest methodsFor: 'export tests' stamp: 'cwp 4/20/2012 23:15'!
testRequireExplicitExports
	env requireExplicitExports.
	env at: #Griffle put: Object new.
	self assert: env exports isEmpty! !

!EnvironmentTest methodsFor: 'binding tests' stamp: 'cwp 4/20/2012 23:28'!
testUndeclaredBindingMoved
	| assoc |
	assoc := env bindingOf: #Griffle.
	env at: #Griffle put: value.
	self assert: (env at: #Griffle) == value! !

!EnvironmentTest methodsFor: 'binding tests' stamp: 'cwp 4/20/2012 19:27'!
testUndeclaredBindingPreserved
	| assoc |
	assoc := env bindingOf: #Griffle.
	env at: #Griffle put: Object new.
	self assert: (env bindingOf: #Griffle) == assoc! !

!EnvironmentTest methodsFor: 'binding tests' stamp: 'cwp 4/20/2012 23:28'!
testWriteAndLookup
	| assoc |
	env at: #Griffle put: value.
	assoc := env bindingOf: #Griffle.
	self assert: assoc key == #Griffle.
	self assert: assoc value == value.
	! !

!EnvironmentTest methodsFor: 'binding tests' stamp: 'cwp 4/20/2012 23:28'!
testWriteAndRead
	env at: #Griffle put: value.
	self assert: (env at: #Griffle) == value.! !

!ClassBuilder class methodsFor: '*env-instance creation' stamp: 'cwp 4/20/2012 19:17'!
forEnvironment: anEnvironment
	^ self basicNew initializeWithEnvironment: anEnvironment! !

!ClassBuilder methodsFor: '*env-initialize-release' stamp: 'cwp 4/20/2012 19:18'!
initializeWithEnvironment: anEnvironment
	self initialize.
	environ := anEnvironment! !

Object subclass: #Environment
	instanceVariableNames: 'contents undeclared organization lookup exports'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Env-Core'!

!Environment class methodsFor: 'as yet unclassified' stamp: 'cwp 6/20/2012 17:27'!
default
	^ Smalltalk! !

!Environment methodsFor: 'accessing' stamp: 'cwp 6/20/2012 11:44'!
associationAt: aSymbol
	"Senders of this should probably be using #bindingOf:"
	
	self flag: #review.
	^ contents associationAt: aSymbol! !

!Environment methodsFor: 'accessing' stamp: 'cwp 4/22/2012 08:46'!
associationAt: aSymbol ifAbsent: aBlock
	"Senders of this should probably be using #bindingOf:"
	
	self flag: #review.
	^ contents associationAt: aSymbol ifAbsent: aBlock! !

!Environment methodsFor: 'accessing' stamp: 'cwp 4/20/2012 17:59'!
at: aSymbol
	^ contents at: aSymbol! !

!Environment methodsFor: 'accessing' stamp: 'cwp 4/20/2012 19:31'!
at: aSymbol ifAbsent: aBlock
	^ contents at: aSymbol ifAbsent: aBlock! !

!Environment methodsFor: 'accessing' stamp: 'cwp 4/20/2012 19:29'!
at: aSymbol ifPresent: aBlock
	^ contents at: aSymbol ifPresent: aBlock! !

!Environment methodsFor: 'accessing' stamp: 'cwp 4/20/2012 18:02'!
at: aSymbol put: anObject
	(contents includesKey: aSymbol) ifFalse:
		[contents declare: aSymbol from: undeclared].
	contents at: aSymbol put: anObject.
	^ anObject! !

!Environment methodsFor: 'binding' stamp: 'cwp 6/22/2012 15:23'!
bindingOf: aSymbol
	^ self bindingOf: aSymbol ifAbsent: nil! !

!Environment methodsFor: 'binding' stamp: 'cwp 6/22/2012 09:14'!
bindingOf: aSymbol ifAbsent: aBlock
	lookup do:
		[:dict | 
		(dict includesKey: aSymbol) ifTrue: 
			[^ dict associationAt: aSymbol]].
	^ undeclared associationAt: aSymbol ifAbsent: aBlock! !

!Environment methodsFor: 'configuring' stamp: 'cwp 4/20/2012 23:31'!
export: aSymbol
	exports add: (contents associationAt: aSymbol)! !

!Environment methodsFor: 'accessing' stamp: 'cwp 4/20/2012 22:57'!
exports
	^ exports! !

!Environment methodsFor: 'file-in' stamp: 'cwp 6/19/2012 21:37'!
fileIn: aStream announcing: aString
	(EnvironmentLoader for: self) 
		fileIn: aStream 
		announcing: aString! !

!Environment methodsFor: 'compatibility' stamp: 'cwp 4/20/2012 19:37'!
flushClassNameCache! !

!Environment methodsFor: 'configuring' stamp: 'cwp 4/20/2012 22:56'!
importEnvironment: anEnvironment
	lookup := lookup copyWith: anEnvironment exports! !

!Environment methodsFor: 'configuring' stamp: 'cwp 6/20/2012 16:34'!
importSmalltalk
	lookup := lookup copyWith: Smalltalk globals! !

!Environment methodsFor: 'initialize-release' stamp: 'cwp 6/22/2012 15:22'!
initialize
	| smalltalk |
	undeclared := IdentityDictionary new.
	contents := IdentityDictionary new.
	lookup := {contents}.
	exports := contents.
	organization := SystemOrganizer defaultList: SortedCollection new
.
	smalltalk := SmalltalkImage basicNew.
	smalltalk globals: self.
	contents at: #Smalltalk put: smalltalk.
	contents at: #Undeclared put: undeclared.! !

!Environment methodsFor: 'accessing' stamp: 'cwp 6/22/2012 01:09'!
keyAtIdentityValue: anObject ifAbsent: aBlock
	^ contents keyAtIdentityValue: anObject ifAbsent: aBlock! !

!Environment methodsFor: 'compatibility' stamp: 'cwp 4/20/2012 19:44'!
organization
	^ organization! !

!Environment methodsFor: 'configuring' stamp: 'cwp 4/20/2012 23:37'!
requireExplicitExports
	exports == contents ifTrue: 
		[exports := IdentityDictionary new]! !

!Environment methodsFor: 'compatibility' stamp: 'cwp 4/20/2012 19:41'!
scopeFor: aSymbol from: lower envtAndPathIfFound: aBlock
	^ (contents includesKey: aSymbol)
		ifTrue: [aBlock value: self value: String new]
		! !

!Environment methodsFor: 'accessing' stamp: 'cwp 6/22/2012 15:48'!
undeclared
	^ undeclared! !

!Environment methodsFor: 'binding' stamp: 'cwp 6/22/2012 09:26'!
valueOf: aSymbol
	^ self valueOf: aSymbol ifAbsent: nil! !

!Environment methodsFor: 'binding' stamp: 'cwp 6/22/2012 09:26'!
valueOf: aSymbol ifAbsent: aBlock
	lookup do:
		[:dict |
		dict at: aSymbol ifPresent: 
			[:value | ^ value]].
	^ aBlock value! !

Object subclass: #EnvironmentLoader
	instanceVariableNames: 'environment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Env-Core'!

!EnvironmentLoader class methodsFor: 'as yet unclassified' stamp: 'cwp 6/19/2012 21:40'!
for: anEnvironment
	^ self basicNew initializeWithEnvironment: anEnvironment! !

!EnvironmentLoader methodsFor: 'as yet unclassified' stamp: 'cwp 6/19/2012 23:09'!
evaluate: chunk logged: aBoolean
	^ [Compiler evaluate: chunk environment: environment logged: aBoolean]
		on: EnvironmentRequest
		do: [:req | req resume: environment]! !

!EnvironmentLoader methodsFor: 'as yet unclassified' stamp: 'cwp 6/20/2012 16:51'!
fileIn: aStream announcing: aString
	| val |
	self logStart: aStream name.
	aString displayProgressFrom: 0 to: aStream size during: 
		[:bar | 
		[aStream atEnd] whileFalse:
			[bar value: aStream position.
			aStream skipSeparators.
			[val := self fileInChunkFrom: aStream]
				on: InMidstOfFileinNotification
				do: [:ex | ex resume: true].
			aStream skipStyleChunk].
		aStream close].	
	self logEnd: aStream name.
	^ val! !

!EnvironmentLoader methodsFor: 'as yet unclassified' stamp: 'cwp 6/20/2012 17:44'!
fileInChunkFrom: aStream
	| chunk |
	^ (aStream peekFor: $!!)
		ifTrue: [ 
			| reader |
			chunk := aStream nextChunk.
			reader := self evaluate: chunk logged: false.
			reader scanFrom: aStream environment: environment]
		ifFalse: [ 
			chunk := aStream nextChunk.
			aStream checkForPreamble: chunk.
			self evaluate: chunk logged: true ]! !

!EnvironmentLoader methodsFor: 'as yet unclassified' stamp: 'cwp 6/19/2012 21:40'!
initializeWithEnvironment: anEnvironment
	self initialize.
	environment := anEnvironment! !

!EnvironmentLoader methodsFor: 'as yet unclassified' stamp: 'cwp 6/20/2012 16:50'!
logEnd: filename
	"Note:  The main purpose of this banner is to flush the changes file."
	SmalltalkImage current logChange: '----End fileIn of ' , filename , ' into ', environment name, '----'	! !

!EnvironmentLoader methodsFor: 'as yet unclassified' stamp: 'cwp 6/20/2012 16:48'!
logStart: filename
	SmalltalkImage current logChange: '----Start fileIn of ' , filename , ' into ' , environment name , '----'! !

Notification subclass: #EnvironmentRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Env-Core'!
